/***************************************************************************

	FILE NAME: DRVR6812.C

	FILE DESCRIPTION:

	This file is a part of DM6812 driver. It contains functions that
	implement timer, interrupts control and others on DM6812 board.

	PROJECT NAME:  DM6812/DM6912 DOS Driver

	DRIVER VERSION: 1.1

	COMPILER: Borland C++ 3.1

	TARGET: Real-Mode DOS

	Copyright 2003 RTD Embedded Technologies

***************************************************************************/

#include <dos.h>
#include <conio.h>
#define BaseAddress          768     // Base address of your boartd
// Address of Register of board
#define rDigitalPort0        0
#define rDigitalPort1        1
#define rPortDir             2
#define rPortMask            2
#define rPortComp            2
#define rDIOClear            2
#define rDioStatus           3
#define rDIOMode             3
#define rTimerClock0         12
#define rTimerClock1         13
#define rTimerClock2         14
#define rTimerControl        15
#define rClrBoardIrq         16
#define rBoardIrqEnable      16
#define rStatusByte          17

#define FALSE                0
#define TRUE                 1

long int PrevClock;
int ModeReg;

/**************************************************************************
 ClearBoardIrq

***************************************************************************/
void ClearBoardIrq (void) {
	inportb(BaseAddress + rClrBoardIrq);
}

/***************************************************************************
 ClearDIOChip

****************************************************************************/
void ClearDIOChip (int AIOChip ){
// AIOChip' is the number of I/O chip in the range of [0..2].
	switch (AIOChip){
		case 0:
			outportb(BaseAddress + rDIOMode, 0);
			outportb(BaseAddress + rDIOClear, 0);
			break;
		case 1:
			outportb(BaseAddress + rDIOMode + 4, 0);
			outportb(BaseAddress + rDIOClear + 4, 0);
			break;
		case 2:
			outportb(BaseAddress + rDIOMode + 8, 0);
			outportb(BaseAddress + rDIOClear + 8, 0);
			break;
	}
}

/***************************************************************************
 ClearDIOIrq

***************************************************************************/
void ClearDIOIrq (int AIOChip ){
// AIOChip' is the number of I/O chip in the range of [0..2].

	ModeReg = inportb(BaseAddress + rDIOMode + (4 * AIOChip));
	ModeReg &= 0xFC;


	switch (AIOChip){
		case 0:
			outportb (BaseAddress + rDIOMode, ModeReg);
			inportb(BaseAddress + rDIOClear);
			break;
		case 1:
			outportb( BaseAddress + rDIOMode + 4, ModeReg);
			inportb(BaseAddress + rDIOClear);
			break;
		case 2:
			outportb (BaseAddress + rDIOMode + 8, ModeReg);
			inportb(BaseAddress + rDIOClear);
			break;
	}
}

/***************************************************************************
 ClockDivisor

 The ClockDivisor procedure is used to set the divisor of a designated
 counter on the 8254 programmable interval timer (PIT).  This procedure
 assumes that the counter has already been set to receive the least
 significant byte (LSB) of the divisor followed by the most significant
 byte (MSB).
****************************************************************************/
void ClockDivisor (int ATimer , int ADivisor) {
	int  MSB ,LSB;

	LSB = ADivisor % 256;
	MSB = ADivisor /256;
	outportb( BaseAddress + rTimerClock0 +  ATimer, LSB);
	outportb( BaseAddress + rTimerClock0 +  ATimer, MSB);
}

/***************************************************************************
 ClockMode

 The ClockMode procedure is used to set the mode of a designated counter
 on the 8254 programmable interval timer (PIT).
****************************************************************************/
void ClockMode (int ATimer, int AMode ) {
	int TimerMode ;

	TimerMode = (ATimer * 64) + (AMode * 2 ) + 0x30;
	outportb( BaseAddress + rTimerControl, TimerMode);
}

/***************************************************************************
 DoneTimer

 Shut down timer operation initiated by 'InitTimer'.
****************************************************************************/
void DoneTimer(void){

//	 Initialize the timers for high speed to ensure the immediate load.
	ClockMode (0, 2);
	ClockMode (1, 2);
	ClockMode (2, 2);
	ClockDivisor (0, 2);
	ClockDivisor (1, 2);
	ClockDivisor (2, 2);
}

/****************************************************************************
 EnableBoardIrq

****************************************************************************/
void EnableBoardIrq (int AEnable ) {

	outportb( BaseAddress + rBoardIrqEnable, AEnable);

}

/****************************************************************************
 EnableDIOIrq

****************************************************************************/
void EnableDIOIrq (int AIOPort , int AEnable){
// AIOChip' is the number of I/O chip in the range of [0..2].
// AEnable' is the IRQ enable ' DISABLE ' or ' ENABLE '.

	ModeReg = inportb(BaseAddress + rDIOMode + (4 * AIOPort));
	ModeReg &= 0xEF;
	ModeReg |= (AEnable * 16);

	switch (AIOPort) {
		case 0:
			outportb( BaseAddress + rDIOMode, ModeReg);
			break;
		case 2:
			outportb( BaseAddress + rDIOMode + 4, ModeReg);
			break;
		case 4:
			outportb( BaseAddress + rDIOMode + 8, ModeReg);
			break;
	}
}

/***************************************************************************
 InitBoard

 The InitBoard() function loads the board settings from the DRVR3110.INI file
 resets the board and clears the FIFO. These two step of initialization is
 important after power up.
****************************************************************************/

void InitBoard (void) {
	ClearDIOChip(0);
	ClearDIOChip(1);
	ClearDIOChip(2);
	ClearBoardIrq();
}

/***************************************************************************
 InitTimer

 The InitTimer function is used to set the user clock to generate the
 designated output rate. The order of programming the timers are important
 to ensure loading the timers fast enough.  The maximum attainable rate
 this function, as written, is 1000 Hz although you can easily change
 this by  adjusting the divisors accordingly.
****************************************************************************/
void InitTimer (int ARate){
// 'ARate' defines the output frequency in 'Hz'.

	if ((ARate < (1000 / 65536L)) | (1000 < ARate))
		cprintf("Invalid parameter in 'InitTimer'.\r\n") ;

	ClockMode( 2, 2);
	ClockDivisor (2, 2000 / ARate);

	ClockMode (1, 2);
	ClockDivisor (1, 2000);

	ClockMode (0, 2);
	ClockDivisor (0, 2);

	PrevClock = 0xFFFF;
}

/***************************************************************************
 IrqStatus%
 Reads the status Register
****************************************************************************/
int IrqStatus(void) {

		return(inportb(BaseAddress + rStatusByte));

}

/***************************************************************************
 isClockDone%

 The isClockDone% function returns TRUE if the designated counter on the
 8254 programmable interval timer (PIT) has overflowed. The function uses
 the "counter latch command" to store the 16 bit counter register for the
 separated reading (low byte first, high byte next). The function stores
 the previous state of the counter register in the 'PrevClock' global
 variable, and returns TRUE, when the current counter register is larger
 than the previous one, i.e. the counter is loaded back. This ensures
 the recognition of the counter owerflow even of high clock rates.
****************************************************************************/
int isClockDone(void) {
	int  MSB ,LSB;
	long CurrentClock;


//	 Counter latch command to store the current counter register.
	outportb( BaseAddress + rTimerControl, 0x80);

//	 Reading the current contents of the counter register.
	LSB = inportb(BaseAddress + rTimerClock2);
	MSB = inportb(BaseAddress + rTimerClock2);
	CurrentClock = 256 * MSB + LSB;

	if (CurrentClock > PrevClock) {
	 PrevClock = CurrentClock;
	 return(TRUE);
	 }
	else {
	 PrevClock = CurrentClock;
	 return(FALSE);
	}
}

/****************************************************************************
 LoadDIOBitDir
 Load a Bit programable IOport. Ports 0,2,4 are Bit programable on the DM6812
****************************************************************************/
void LoadDIOBitDir (int AIOPort, int ADir7,int ADir6, int ADir5, int ADir4,
						  int ADir3,int ADir2,int ADir1,int ADir0) {
// AIOPort' is the number of I/O port in the range of [0,2, or 4].
// ADir' is the direction ' IN ' or ' OUT '.

	ModeReg = inportb(BaseAddress + rDIOMode + (2 * AIOPort));
	ModeReg &= 0xFC;
	ModeReg |=0x1;

	switch(AIOPort){
		case 0:
			outportb( BaseAddress + rDIOMode, ModeReg);
			outportb( BaseAddress + rPortDir, ((ADir7 * 128) + (ADir6 * 64) + (ADir5 * 32) + (ADir4 * 16) + (ADir3 * 8) + (ADir2 * 4) + (ADir1 * 2) + (ADir0 * 1)));
			break;
		case 2:
			outportb( BaseAddress + rDIOMode + 4, ModeReg);
			outportb( BaseAddress + rPortDir + 4, ((ADir7 * 128) + (ADir6 * 64) + (ADir5 * 32) + (ADir4 * 16) + (ADir3 * 8) + (ADir2 * 4) + (ADir1 * 2) + (ADir0 * 1)));
			break;
		case 4:
			outportb( BaseAddress + rDIOMode + 8, ModeReg);
			outportb( BaseAddress + rPortDir + 8, ((ADir7 * 128) + (ADir6 * 64) + (ADir5 * 32) + (ADir4 * 16) + (ADir3 * 8) + (ADir2 * 4) + (ADir1 * 2) + (ADir0 * 1)));
			break;
	}

}

/****************************************************************************
 LoadDIOComp

****************************************************************************/
void LoadDIOComp (int AIOChip,int AData){
//  AIOChip' is the number of I/O chip in the range of [0..2].
//  AData' is the binary value to send out to the 'Compare Register'.

	ModeReg = inportb(BaseAddress + rDIOMode + (4 * AIOChip));
	ModeReg &= 0xFC;
	ModeReg |=3;

	switch (AIOChip){
		case 0:
			outportb(BaseAddress + rDIOMode, ModeReg);
			outportb(BaseAddress + rPortComp, AData);
			break;
		case 1:
			outportb(BaseAddress + rDIOMode + 4, ModeReg);
			outportb(BaseAddress + rPortComp + 4, AData);
			break;
		case 2:
			outportb(BaseAddress + rDIOMode + 8, ModeReg);
			outportb(BaseAddress + rPortComp + 8, AData);
			break;
	}
}

/****************************************************************************
 LoadDIOMask

****************************************************************************/
void LoadDIOMask (int AIOChip,int AData) {
// AIOChip' is the number of I/O chip in the range of [0..2].
// AData' is the binary value to send out to the 'Mask Register'.

	ModeReg = inportb(BaseAddress + rDIOMode + (4 * AIOChip));
	ModeReg &= 0xFC;
	ModeReg |= 0x2;

	switch(AIOChip) {
		case 0:
			outportb( BaseAddress + rDIOMode, ModeReg);
			outportb( BaseAddress + rPortMask, AData);
			break;
		case 1:
			outportb( BaseAddress + rDIOMode + 4, ModeReg);
			outportb( BaseAddress + rPortMask + 4, AData);
			break;
		case 2:
			outportb( BaseAddress + rDIOMode + 8, ModeReg);
			outportb( BaseAddress + rPortMask + 8, AData);
			break;
	}
}

/****************************************************************************
 LoadDIOPortDir

 Load a Port programable IOport. Ports 1,3,5 are Port programable on the DM6812
****************************************************************************/
void LoadDIOPortDir (int AIOPort,int ADir){
// AIOPort' is the number of I/O port in the range of [1,3, or 5].
// ADir' is the direction ' IN ' or ' OUT '.

	ModeReg = inportb(BaseAddress + rDIOMode + (2 * (AIOPort - 1)));
	ModeReg &= 0xFB;
	ModeReg |= (ADir * 4);

	switch (AIOPort){
		case 1:
			outportb( BaseAddress + rDIOMode, ModeReg);
			break;
		case 3:
			outportb( BaseAddress + rDIOMode + 4, ModeReg);
			break;
		case 5:
			outportb( BaseAddress + rDIOMode + 8, ModeReg);
			break;
	}

}

/***************************************************************************
 ReadClock&

****************************************************************************/
unsigned  ReadClock (int ATimer){
// ATimer' is the number of the Timer in the range of 0..2.

	unsigned MSB,LSB;
	unsigned  time;

//	 Counter latch command to store the current counter register.
	outportb( BaseAddress + rTimerControl, (ATimer * 64));

//  Reading the current contents of the counter register.
	LSB = inportb(BaseAddress + rTimerClock0 + ATimer);
	MSB = inportb(BaseAddress + rTimerClock0 + ATimer);
	time = 256u *(unsigned )MSB + (unsigned )LSB;
	return(time);

}

/***************************************************************************
 ReadDIO%

 The ReadDIO% function returns the value of the specified digital
 input port.  Each digital input line is represented by a bit of the
 return value. Digital in 0 is bit 0, digital in 1 is bit 1, and so on.
****************************************************************************/
int ReadDIO (int AIOPort) {
// AIOPort' is the number of I/O port in the range of 0..5.

	switch (AIOPort) {
		case 0: return (inportb(BaseAddress + rDigitalPort0));
		case 1: return (inportb(BaseAddress + rDigitalPort1));
		case 2: return inportb((BaseAddress + rDigitalPort0 + 4));
		case 3: return inportb((BaseAddress + rDigitalPort1 + 4));
		case 4: return inportb((BaseAddress + rDigitalPort0 + 8));
		case 5: return inportb((BaseAddress + rDigitalPort1 + 8));
		default: return(0);
		}
}

/****************************************************************************
 SelectDIOClock

****************************************************************************/
void SelectDIOClock (int AIOPort,int AClock){
// AIOChip' is the number of I/O port in the range of [0..2].
// AClock' is the Clock mode ' SYSTEM ' or ' PROGRAMMABLE '.

	ModeReg = inportb(BaseAddress + rDIOMode + (4 * AIOPort));
	ModeReg &= 0xDF;
	ModeReg |= (AClock * 32);

	switch (AIOPort){
		case 0:
			outportb( BaseAddress + rDIOMode, ModeReg );
			break;
		case 2:
			outportb( BaseAddress + rDIOMode + 4, ModeReg );
			break;
		case 4:
			outportb( BaseAddress + rDIOMode + 8, ModeReg );
			break;
	}

}

/****************************************************************************
 SetDIOIrqMode

****************************************************************************/
void SetDIOIrqMode (int AIOChip,int AMode) {
// AIOChip' is the number of I/O port in the range of [0..2].
// AMode' is the IRQ mode ' EVENT ' or ' MATCH '.

	ModeReg = inportb(BaseAddress + rDIOMode + (4 * AIOChip));
	ModeReg &= 0xF7;
	ModeReg |= (AMode * 8);

	switch (AIOChip){
		case 0:
			outportb( BaseAddress + rDIOMode, ModeReg);
			break;
		case 1:
			outportb( BaseAddress + rDIOMode + 4, ModeReg);
			break;
		case 2:
			outportb( BaseAddress + rDIOMode + 8, ModeReg);
			break;
	}

}

/***************************************************************************
 TestTimer%

 The TestTimer% function returns TRUE, if the timer of the board works fine.
 Testing the timer is important before running some of the example programs
 to asub infinite loops waiting for timer tick.
****************************************************************************/
int TestTimer(void){
	int i,d;

	InitTimer(20);                      // Set timer for 20.0 Hz.
	i = 0 ;                              // Wait 200 ms for the timer.
	while ((!isClockDone) & (i < 20))
	 for ( d = 1 ;d < 100;d++) {        // delay a bit
	 i++ ;
	}
	DoneTimer();                        // Shut down timer operation.
	return(i < 20);                     //Return TRUE if the timer works.
}

char* TitleString(void){
	return("DM6812 sample program.");
}

/****************************************************************************
 WriteDIO

 The WriteDIO function writes 'Data' to the specified digital output port.
****************************************************************************/
void WriteDIO (int AIOPort,int AData) {
// AIOPort' is the number of I/O port in the range of [0..5].
// AData' is the binary value to send out on 'IOPort'.

	switch (AIOPort) {
		case 0:
			outportb( BaseAddress + rDigitalPort0, AData);
			break;
		case 1:
			outportb( BaseAddress + rDigitalPort1, AData);
			break;
		case 2:
			outportb( BaseAddress + rDigitalPort0 + 4, AData);
			break;
		case 3:
			outportb( BaseAddress + rDigitalPort1 + 4, AData);
			break;
		case 4:
			outportb( BaseAddress + rDigitalPort0 + 8, AData);
			break;
		case 5:
			outportb( BaseAddress + rDigitalPort1 + 8, AData);
			break;
	}
}


/****************************************************************************
 InitHostIT()

 The InitHostIT() function saves the old interrupt vector, set the new
 interrupt vector and unmask the IRQ channel in the mask register of the
 interrupt controller. InitHostIT() works differently for XT and AT interrupt
 channels.
****************************************************************************/

#ifdef __cplusplus
// Original interrupt storage C++ style
void interrupt (*OldINT) (...);

void InitHostIT(void interrupt (*IntVect) (...),char IRQNum)
#else
// Original interrupt storage C style
void interrupt (*OldINT) (void);

void InitHostIT(void interrupt (*IntVect) (void),char IRQNum)
#endif
{
  char ITVector, IMR;

  disable();                           // Don't want int called during revector
  if (IRQNum < 8) {
	 ITVector = IRQNum + 8;             // XT interrupt channel
	 OldINT = getvect(ITVector);        // Save startup vector
	 setvect(ITVector, IntVect);        // Set the vector
	 IMR = inportb(0x21);               // Read in current IMR
	 IMR &= ~(1 << IRQNum);             // Clear desired bit
	 outportb(0x21, IMR);               // out new IMR
  }
  else {
	 ITVector = IRQNum + 0x68;          // AT interrupt channel
	 OldINT = getvect(ITVector);        // Save startup vector
	 setvect(ITVector, IntVect);        // Set the vector
	 IMR = inportb(0xA1);               // Read in current IMR
	 IMR &= ~(1 << (IRQNum-8));         // Clear desired bit
	 outportb(0xA1, IMR);               // out new IMR
  }
  enable();                            // Reenable interrupts
} //InitHostIT

/****************************************************************************
 RestoreHostIT()

 The RestoreHostIT() function masks the IRQ channel in the mask register of
 the interrupt controller and restores the old interrupt vector.
 RestoreHostIT() works differently for XT and AT interrupt channels.
****************************************************************************/

void RestoreHostIT(char IRQNum)
{
  char ITVector, IMR;

  disable();                           // Don't want int called during revector
  if (IRQNum < 8) {
	 ITVector = IRQNum + 8;             // XT interrupt channel
	 IMR = inportb(0x21);               // Read in current IMR
	 IMR |= 1 << IRQNum;                // Set desired bit
	 outportb(0x21, IMR);               // out new IMR
  }
  else {
	 ITVector = IRQNum + 0x68;          // AT interrupt channel
	 IMR = inportb(0xA1);               // Read in current IMR
	 IMR |= 1 << (IRQNum-8);            // Set desired bit
	 outportb(0xA1, IMR);               // out new IMR
  }
  setvect(ITVector, OldINT);           // Restore the old vector
  enable();                            // Reenable interrupts
} //RestoreHostIT

